@@ -761,7 +761,7 @@
     (inst sra fixnum n-fixnum-tag-bits digit)))
 
 (define-vop (bignum-floor)
-  (:translate sb!bignum:%floor)
+  (:translate sb!bignum:%bigfloor)
   (:policy :fast-safe)
   (:args (num-high :scs (unsigned-reg))
          (num-low :scs (unsigned-reg))
