@@ -287,6 +287,11 @@
          (sc-number-or-lose 'immediate)
          nil))))
 
+(!def-vm-support-routine boxed-immediate-sc-p (sc)
+  (or (eql sc (sc-number-or-lose 'zero))
+      (eql sc (sc-number-or-lose 'null))
+      (eql sc (sc-number-or-lose 'immediate))))
+
 ;;; A predicate to see if a character can be used as an inline
 ;;; constant (the immediate field in the instruction used is sixteen
 ;;; bits wide, which is not the same as any defined subtype of
@@ -376,23 +381,23 @@
          (t (values :default nil))))
       ;; FIXME: can handle MIN and MAX here
       (sb!kernel:%ldb
-       (cond
-         ((or (valid-funtype '((constant-arg (integer 1 29))
-                               (constant-arg (integer 0 29))
-                               fixnum)
-                             'fixnum)
-              (valid-funtype '((constant-arg (integer 1 29))
-                               (constant-arg (integer 0 29))
-                               (signed-byte 32))
-                             'fixnum)
-              (valid-funtype '((constant-arg (integer 1 29))
-                               (constant-arg (integer 0 29))
-                               (unsigned-byte 32))
-                             'fixnum))
-          (values :transform
-                  '(lambda (size posn integer)
-                    (%%ldb integer size posn))))
-         (t (values :default nil))))
+       (flet ((validp (type width)
+                (and (valid-funtype `((constant-arg (integer 1 29))
+                                      (constant-arg (mod ,width))
+                                      ,type)
+                                    'fixnum)
+                     (destructuring-bind (size posn integer)
+                         (sb!c::basic-combination-args node)
+                       (declare (ignore integer))
+                       (<= (+ (sb!c::lvar-value size)
+                              (sb!c::lvar-value posn))
+                           width)))))
+         (if (or (validp 'fixnum 29)
+                 (validp '(signed-byte 32) 32)
+                 (validp '(unsigned-byte 32) 32))
+             (values :transform '(lambda (size posn integer)
+                                  (%%ldb integer size posn)))
+             (values :default nil))))
       (t (values :default nil)))))
 
 (defun primitive-type-indirect-cell-type (ptype)
