@@ -323,6 +323,11 @@
      (if (static-symbol-p value)
          (sc-number-or-lose 'immediate)
          nil))))
+
+(!def-vm-support-routine boxed-immediate-sc-p (sc)
+  (or (eql sc (sc-number-or-lose 'zero))
+      (eql sc (sc-number-or-lose 'null))
+      (eql sc (sc-number-or-lose 'immediate))))
 
 ;;;; function call parameters
 
