@@ -53,7 +53,7 @@
 #-sb-unicode
 (progn
   (test-util:report-test-status)
-  (sb-ext:quit :unix-status 104))
+  (sb-ext:exit :code 104))
 
 ;;; Test UTF-8 writing and reading of 1, 2, 3 and 4 octet characters with
 ;;; all possible offsets. Tests for buffer edge bugs. fd-stream buffers are
